import 'package:flutter/material.dart';
import 'store.dart';
import 'listener_client.dart';

abstract class ListenerWidgetState<T> extends State<StatefulWidget> {

  void _onUpdateCallBack() {
    setState(() {});
  }

  List<ListenerClient> getInitialListenerClients();

  List<ListenerClient> _copyOfUsedListenerClients;

  void _startListening() {
    for (ListenerClient client in _copyOfUsedListenerClients) {
      client.updateSubscriptionGenerator();
      Store store = client.getStore();
      store.getListener().beginListening(_onUpdateCallBack, client);
    }
  }

  void _stopListening() {
    for (ListenerClient client in _copyOfUsedListenerClients) {
      Store store = client.getStore();
      store.getListener().terminateSubscription(client);
    }
  }

  void updateListenerClients(List<ListenerClient> newListenerClients) {
    _stopListening();
    _copyOfUsedListenerClients = newListenerClients;
    _startListening();
  }


  @override
  void initState() {
    super.initState();
    _copyOfUsedListenerClients = getInitialListenerClients();
    _startListening();
  }

  @override
  void dispose() {
    super.dispose();
    _stopListening();
  }

  bool allDataLoaded() {
    return _copyOfUsedListenerClients.every((ListenerClient client) => client.getStore().isLoaded());
  }
}